<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/1/16
 * Time: 6:51 PM
 */

namespace qfq\store;

use qfq\exceptions\CodeException;
use qfq;


require_once(__DIR__ . '/../../qfq/helper/KeyValueStringParser.php');
require_once(__DIR__ . '/../../qfq/helper/Sanatize.php');
require_once(__DIR__ . '/../../qfq/Constants.php');


/*
 * Stores:
 * - SIP
 * - webVar
 * - record
 * - form
 * - formElement
 */

class Store {

    private static $instance = null;

    private static $raw = array();
    private static $cache = array();
    private static $defaultSanatizeClass = array();

    /**
     * Design Pattern: Singleton - class 'store' should only be instanced once.
     */
    private function __construct($bodytext = '') {

        self::$defaultSanatizeClass = [
            T3_BODYTEXT_DEBUG_LOAD => SANATIZE_DIGIT,
            T3_BODYTEXT_DEBUG_SAVE => SANATIZE_DIGIT,

            CLIENT_SIP => SANATIZE_DIGIT,
            CLIENT_RECORD_ID => SANATIZE_DIGIT,
            CLIENT_KEY_SEM_ID => SANATIZE_DIGIT,
            CLIENT_KEY_SEM_ID_USER => SANATIZE_DIGIT,
            CLIENT_PAGE_ID => SANATIZE_DIGIT,
            CLIENT_PAGE_TYPE => SANATIZE_DIGIT,
            CLIENT_PAGE_LANGUAGE => SANATIZE_DIGIT,
            CLIENT_FORM => SANATIZE_ALNUMX,

            SYSTEM_DBUSER => SANATIZE_ALNUMX,
            SYSTEM_DBSERVER => SANATIZE_ALNUMX,
            SYSTEM_DBPW => SANATIZE_ALL,
            SYSTEM_DB => SANATIZE_ALNUMX,
            SYSTEM_TESTDB => SANATIZE_ALNUMX,
            SYSTEM_SESSIONNAME => SANATIZE_ALNUMX,
            SYSTEM_DBH => SANATIZE_ALL
        ];

        self::fillSystemStore();
        self::fillStoreT3Bodytext($bodytext);
        self::fillStoreClient();
    }

    /**
     * @throws CodeException
     * @throws qfq\exceptions\UserException
     */
    private function fillSystemStore() {
        try {
            $config = parse_ini_file(QFQ_INI, false);
        } catch (\Exception $e) {
            throw new qfq\exceptions\UserException ("Error read file " . QFQ_INI . ": " . $e->getMessage(), ERROR_READ_FILE);
        }
        self::setVarArray($config, STORE_SYSTEM);
    }

    /**
     * @param array $dataArray
     * @param $store
     * @throws CodeException
     */
    public function setVarArray(array $dataArray, $store) {

        if (isset(self::$raw[$store]) && count(self::$raw[$store]) > 0) {
            throw new CodeException("Raw values already been copied to store '$store'. Do this only one time.", ERROR_VALUE_ALREADY_CODPIED);
        }

        self::$raw[$store] = $dataArray;
    }

    /**
     * @param $bodytext
     * @throws CodeException
     */
    private function fillStoreT3Bodytext($bodytext) {
        $keyValue = new \qfq\helper\KeyValueStringParser("=", "\n");

        self::setVarArray($keyValue->parse($bodytext), STORE_T3_BODYTEXT);
    }

    /**
     * @throws CodeException
     */
    private function fillStoreClient() {
        // copy GET and POST Parameter. POST Parameter have a higher priority.
        $arr = $_GET;
        foreach ($_POST as $key => $value) {
            $arr[$key] = $value;
        }
        self::setVarArray($arr, STORE_CLIENT);
    }

    /**
     *
     * @return Store
     */
    public static function getInstance($bodytext = '', $phpunit = false) {

        if ($phpunit) {
            if (self::$instance === null) {
                unset(self::$instance);
                self::$instance = null;
            }
        }

        // Design Pattern: Singleton
        if (self::$instance === null) {
            self::$instance = new Store($bodytext);
        }
        return self::$instance;
    }

    /**
     * @param $key
     * @param $value
     * @param $store
     * @throws CodeException
     */
    public static function setVar($key, $value, $store) {

        self::$raw[$store][$key] = $value;
        // Invalidate cache to force new sanatize
        self::$cache[$store][$key] = null;

    }

    /**
     * Cycles throught all stores in $useStore.
     * First match will return the found value.
     * During cycling:, fill cache and sanatize a raw value.
     *
     * @param string $key
     * @param string $useStores f.e.: 'FSRD'
     * @param string $sanatizeClass
     * @return string a) if found: value, b) false
     */
    public function getVar($key, $useStores = USE_STORE_DEFAULT, $sanatizeClass = '') {

        // no store specifed?
        if ($useStores === "") {
            return false;
        }

        while ($useStores !== false) {

            $store = substr($useStores, 0, 1); // next store
            $useStores = substr($useStores, 1); // shift left remaining stores

            $cacheVal = isset(self::$cache[$store][$key]) ? self::$cache[$store][$key] : null;
            $rawVal = isset(self::$raw[$store][$key]) ? self::$raw[$store][$key] : null;

            // Already cached?
            if (isset($cacheVal) && $cacheVal !== false)
                return $cacheVal;

            // No value in store: try next store
            if ($cacheVal === false) {
                continue;
            }

            // No value in raw? set cache to 'nothing found' and try next store
            if (!isset($rawVal)) {
                self::$cache[$store][$key] = false;
                continue;
            }

            // get final sanatizeClass
            if ($sanatizeClass === '') {
                $sanatizeClass = isset(self::$defaultSanatizeClass[$key]) ? self::$defaultSanatizeClass[$key] : SANATIZE_DEFAULT;
            }

            // sanatize
//            $value =  \qfq\Sanatize::sanatize($rawVal, $sanatizeClass);
//            self::$cache[$store][$key] = $value;
//
//            if(!isset(self::$cache[$store])) {
//                self::$cache[$store]=array();
//                self::$cache[$store][$key] = $value;
//            }
            self::$cache[$store][$key] = \qfq\Sanatize::sanatize($rawVal, $sanatizeClass);
            return self::$cache[$store][$key];

        }
        return false;

    }
}




